#pragma once
// as_scripting.
// Written by Dominque Douglas
// Copyright 2013 Dominque Douglas


// Include the definitions of the script library and the add-ons we'll use.
// The project settings may need to be configured to let the compiler where
// to find these headers. Don't forget to add the source modules for the
// add-ons to your project as well so that they will be compiled into the 
// application.
#include <angelscript.h>
#include "scriptbuilder/scriptbuilder.h"

#include "XACTGameScriptConstants.h"

//#include "game.h"

// Access Masks -------------------------------------------------------------
const unsigned int ScriptInterfaceMask_SetupOnly	= 0x01; // interface can only be called by modules with this mask
const unsigned int ScriptInterfaceMask_Gameplay		= 0x02; // interface can only be called by modules with this mask
const unsigned int ScriptInterfaceMask_All			= ScriptInterfaceMask_SetupOnly | ScriptInterfaceMask_Gameplay; // interface can only be called both modules types

// enumerations -------------------------------------------------------------
enum ScriptFunctionIDs
{
	Function_InitApp = 0,
	Function_FireAmmo,
	Function_HandleAmmoAI,
	Function_HandleDroidAI,
	Function_CreateDroid
};

const unsigned int max_script_functions = 5;

// Structures ---------------------------------------------------------------
struct ScriptContextData
{
	asIScriptContext *ctx;
	asIScriptFunction *script_functions[max_script_functions];

	CXACTGameScriptConstants constants;

	int PrepareFunction(ScriptFunctionIDs func_id)
	{
		// I'm no longer checking for a valid context here. It's up to the application writer to ensure 
		// that the context is valid before calling this
		return ctx->Prepare(script_functions[func_id]);
	}

	int ExecuteFunction()
	{
		// I'm no longer checking for a valid context here. It's up to the application writer to ensure 
		// that the context is valid before calling this
		int result = ctx->Execute();
		if( result != asEXECUTION_FINISHED )
		{
			// The execution didn't complete as expected. Determine what happened.
			if( result == asEXECUTION_EXCEPTION )
			{
				// An exception occurred, let the script writer know what happened so it can be corrected.
				MessageBoxA(NULL, ctx->GetExceptionString(), "An exception occurred.", MB_OK);
				return -1;
			}
		}
		return result;
	}
};

// Forward Declarations -----------------------------------------------------
int StartScriptingSystem(asIScriptEngine *&scriptengine, ScriptContextData &contextdata);
void ShutdownScriptingSystem(asIScriptEngine *&scriptengine, asIScriptContext *&ctx);

int RegisterGameInterface(asIScriptEngine *scriptengine);
int RegisterEnumGAME_MODE(asIScriptEngine *scriptengine);

int LoadScript(asIScriptEngine *scriptengine, ScriptContextData &contextdata);